@@ -43,6 +43,7 @@ class InternalBuilderBackend():
             self._loopback_setup()
             self._create_filesystems()
             self._mount_filesystems()
+            self._setup_extra_storage()
             self._debootstrap()
             self._set_hostname()
             self._lock_root_user()
@@ -78,9 +79,11 @@ class InternalBuilderBackend():
             return library.create_temp_image(self.state,
                                              self.builder.image_file)
 
+        size = utils.add_disk_sizes(self.builder.arguments.image_size,
+                                    self.builder.extra_storage_size)
+        size = utils.add_disk_sizes(size, '100M')  # Buffer
         return library.create_ram_directory_image(
-            self.state, self.builder.image_file,
-            self.builder.arguments.image_size)
+            self.state, self.builder.image_file, size)
 
     def _create_empty_image(self):
         """Create an empty disk image to create parititions in."""
@@ -140,6 +143,22 @@ class InternalBuilderBackend():
         if self.builder.firmware_filesystem_type:
             library.mount_filesystem(self.state, 'firmware', 'boot/firmware')
 
+    def _setup_extra_storage(self):
+        """Setup some extra storage for root filesystem.
+
+        btrfs runs out of space even when the data stored is not even half of
+        the available space. This is due to it needing re-balance operation. It
+        may not be possible to run the re-balance during a heavy operation such
+        as installing a package with lot of dependencies (freedombox). Adding
+        an additional disk is a good work around for that. The additional
+        device will be removed during cleanup and filesystem will be
+        re-balanced.
+
+        """
+        library.setup_extra_storage(self.state,
+                                    self.builder.root_filesystem_type,
+                                    self.builder.extra_storage_size)
+
     def _mount_additional_filesystems(self):
         """Mount extra filesystems: dev, devpts, sys and proc."""
         library.mount_filesystem(self.state, '/dev', 'dev', is_bind_mount=True)
@@ -160,10 +179,9 @@ class InternalBuilderBackend():
             self._get_filesystem_packages() + \
             self._get_extra_packages()
 
-    @staticmethod
-    def _get_basic_packages():
+    def _get_basic_packages(self):
         """Return a list of basic packages for all king of images."""
-        return ['firmware-ath9k-htc']
+        return self.builder.packages
 
     def _get_kernel_packages(self):
         """Return package needed for kernel."""
@@ -181,6 +199,9 @@ class InternalBuilderBackend():
             return ['grub-pc']
 
         if self.builder.boot_loader == 'u-boot':
+            if self.builder.architecture == 'arm64':
+                return ['u-boot-sunxi', 'u-boot-tools']
+
             return ['u-boot', 'u-boot-tools']
 
         raise NotImplementedError
