@@ -235,7 +235,8 @@ def create_filesystem(device, filesystem_type):
     # update-grub using root=/dev/mapper/loop0p1 instead of root=UUID=<uuid>
     # when creating grub.cfg. This results in an unbootable image. Force udev
     # events as a workaround.
-    run(['udevadm', 'trigger', '--settle', device])
+    run(['udevadm', 'trigger', device])
+    run(['udevadm', 'settle'])
 
 
 def mount_filesystem(state,
@@ -276,6 +277,39 @@ def unmount_filesystem(device, mount_point, is_bind_mount, ignore_fail=False):
     run(['umount', mount_point], ignore_fail=ignore_fail)
 
 
+def setup_extra_storage(state, file_system_type, size):
+    """Add an extra storage device to a btrfs filesystem."""
+    if file_system_type != 'btrfs':
+        return
+
+    extra_storage_file = state['image_file'] + '.extra'
+    mount_point = state['mount_point']
+    logger.info('Adding extra storage to file system %s', mount_point)
+
+    run(['qemu-img', 'create', '-f', 'raw', extra_storage_file, size])
+    output = run(['losetup', '--show', '--find', extra_storage_file])
+    loop_device = output.decode().strip()
+    run(['btrfs', 'device', 'add', loop_device, mount_point])
+
+    schedule_cleanup(state, cleanup_extra_storage, state, loop_device,
+                     extra_storage_file)
+
+
+def cleanup_extra_storage(state, loop_device, extra_storage_file):
+    """Remove the extra storage added to a btrfs filesystem and balance it."""
+    mount_point = state['mount_point']
+    logger.info('Removing extra storage from file system %s', mount_point)
+
+    # Remove the extra storage device from btrfs filesystem
+    run(['btrfs', 'device', 'remove', loop_device, mount_point])
+    run(['losetup', '--detach', loop_device])
+    run(['rm', '-f', extra_storage_file])
+
+    # Re-balance the filesystem
+    run(['btrfs', 'balance', 'start', '-musage=20', mount_point])
+    run(['btrfs', 'balance', 'start', '-dusage=20', mount_point])
+
+
 def qemu_debootstrap(state, architecture, distribution, variant, components,
                      packages, mirror):
     """Debootstrap into a mounted directory."""
@@ -438,15 +472,24 @@ deb-src {mirror} {distribution} {components}
     updates_template = '''
 deb {mirror} {distribution}-updates {components}
 deb-src {mirror} {distribution}-updates {components}
-
+'''
+    old_security_template = '''
 deb http://security.debian.org/debian-security/ {distribution}/updates {components}
 deb-src http://security.debian.org/debian-security/ {distribution}/updates {components}
+'''
+    security_template = '''
+deb http://security.debian.org/debian-security/ {distribution}-security {components}
+deb-src http://security.debian.org/debian-security/ {distribution}-security {components}
 '''
     file_path = path_in_mount(state, 'etc/apt/sources.list')
     with open(file_path, 'w') as file_handle:
         file_handle.write(basic_template.format(**values))
         if distribution not in ('sid', 'unstable'):
             file_handle.write(updates_template.format(**values))
+            if distribution in ('bullseye', 'testing'):
+                file_handle.write(security_template.format(**values))
+            else:  # stable/buster
+                file_handle.write(old_security_template.format(**values))
 
     run_in_chroot(state, ['apt-get', 'update'])
     run_in_chroot(state, ['apt-get', 'clean'])
